const fetch = require('node-fetch')
const { GuildAction } = require('./guild_action')

/**
 * @typedef {Object} SearchResult
 * @property {string} url The url the resource can be accessed from
 * @property {string} title The title of the resource
 * @property {string} length A string representing the duration of the resource
 * @property {string} source The source where the resource comes from
 */

/**
 * @class
 */
class ResourceManager {
  #gmanager
  #awaiting_answers

  constructor(gmanager) {
    this.#gmanager = gmanager
    this.#awaiting_answers = []
  }

  /**
   * @param {string} search The search pattern that will be sent to Youtube
   * @return {SearchResult[]} An array of objects representing the search results. <code>source</code> is set to <code>"youtube"</code>
   */
  async #search_youtube(search) {
    try {
      const req = await fetch('https://www.youtube.com/results?search_query='
                              + encodeURIComponent(search))
      const body = await req.text()
      const json = JSON.parse(body.match(/ytInitialData = (.*);<\/script>/)[1])
      const results = json.contents.twoColumnSearchResultsRenderer.primaryContents.sectionListRenderer.contents[0].itemSectionRenderer.contents
      return results
        .filter(e => e.videoRenderer)
        .map(e => {
          return {url: 'https://www.youtube.com/watch?v=' + e.videoRenderer.videoId,
                  title: e.videoRenderer.title.runs[0].text,
                  length: e.videoRenderer.lengthText.simpleText,
                  source: 'youtube'}
        })
    } catch(e) {
      return []
    }
  }

  async discover_youtube_full_title(url) {
    const req = await fetch(url)
    const body = await req.text()
    const json = JSON.parse(body.match(/ytInitialData = (.*);<\/script>/)[1])
    const video_infos = json.contents.twoColumnWatchNextResults.results.results.contents[0].videoPrimaryInfoRenderer
    return video_infos.title.runs[0].text
  }

  /**
   * Do a research and send the search results back to the Discord channel it's been asked for
   * @param {object} message An object representing the incoming message
   * @param {string[]} params The arguments of the message
   * @return {SearchResult[]} An array of objects representing the search results.
   */
  async search({ message, params }) {
    const json_res = await this.#search_youtube(params.join(' '))
    if (json_res.length === 0) {
      message.channel.send(':warning: Something went wrong. Please try again')
    } else {
      const text_res = json_res
            .slice(0, 10)
            .reduce((acc, video, i) => acc + (i + 1) + ' ' + video.title + ' (' + video.length + ')\n', '')
      const input_manager = this.#gmanager.get_manager(message.guild.id, 'input')
      const trigger = {
        pattern: /^\d+/,
        target: 'resource',
        action: 'search_confirm',
        ttl: '5 minutes',
        user_shared: false,
        repeat: 1
      }
      this.#awaiting_answers.push({
        trigger: trigger,
        search_results: json_res,
        from: message.member.id
      })
      input_manager.add_trigger(trigger)
      // TODO: 2000 chars limit
      // TODO: find a better syntaxic coloration than js
      message.channel.send('```js\n' + text_res + '```')
    }
    return json_res
  }

  async search_confirm({ message, params }) {
    // TODO: update this so that it can be more relient on the Trigger received
    const id = parseInt(message.content)
    const answer = this.#awaiting_answers
          .filter(a => a.from === message.member.id)
          .slice(-1)[0]
    if (answer === undefined) {
      message.channel.send(':warning: Something went wrong. That\'s not supposed to happen...')
    } else if (answer.search_results[id - 1] === undefined) {
      message.channel.send(':warning: Please choose a number between 1 and ' + answer.search_results.length)
    } else {
      // TODO: calling the add method is a bit ugly, we can probably do better...
      const action = new GuildAction({ message, params: [answer.search_results[id - 1].url] })
      this.#gmanager.get_manager(message.guild.id, 'player').add(action)
      message.react('👌')
    }
  }
}

module.exports = { ResourceManager }
